package org.gridschool.unicore.plugins.boltzmann;

import java.io.File;
import java.util.logging.Level;

import javax.swing.JPanel;

import org.unicore.ajo.ActionGroup;
import org.unicore.ajo.IncarnateFiles;
import org.unicore.ajo.UserTask;
import org.unicore.ajo.ActionGroup.InvalidDependencyException;
import org.unicore.resources.Resource;
import org.unicore.resources.SoftwareResource;
import org.unicore.sets.ResourceEnumeration;
import org.unicore.sets.ResourceSet;

import com.pallas.unicore.client.panels.IPanelProvider;
import com.pallas.unicore.container.GroupContainer;
import com.pallas.unicore.container.UserContainer;
import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.container.errorspec.UError;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.StringTools;

/**
 * A basic example container executing a ApplicationResource and adding
 * an OutcomePanel to the Job Monitor area
 *
 *@author     Ralf Ratering
 *@version    $Id: BoltzmannContainer.java,v 1.5 2003/07/12 16:02:03 rrate Exp $
 */
public class BoltzmannContainer
	extends UserContainer
	implements IPanelProvider {

	// name of our ApplicationResource
	private static final String APPLICATION_NAME = "Boltzmann";

	// input filename in job directory
	private static final String INPUT_FILENAME = "input";

	// id for compatibility between versions
	private static final long serialVersionUID = -4933274194596975934L;

	// default input filename
	private String filename = "BoltzmannInput";

	// local or remote filename for loaded input file
	private File inputFile;

	// input file contents
	private String inputString;

	// additional outcome panel for job monitor area to get sample files
	// make this one transient to prevent it from being serialized
	private transient BoltzmannSamplePanel samplePanel;

	// same for control panel
	private transient BoltzmannControlPanel controlPanel;

	/**
	 *  Constructor
	 *
	 *@param  parentContainer  parent job group
	 */
	public BoltzmannContainer(GroupContainer parentContainer) {
		super(parentContainer);
	}

	/**
	 *  Get the File belonging to the script
	 *
	 *@return    script file object
	 */
	public File getInputFile() {
		return inputFile;
	}

	/**
	 *  Get the script contents as String
	 *
	 *@return    script contents String
	 */
	public String getInputString() {
		return inputString;
	}

	/**
	 *  Build the AJO ActionGroup that will be plugged into parent job group.
	 */
	protected void buildExecuteGroup() {
		logger.info("Building execution ActionGroup");

		// get input file contents as 2D byte array
		byte[][] contents = new byte[1][];
		contents[0] = StringTools.dos2Unix(inputString).getBytes();

		String[] filenames = { INPUT_FILENAME };

		// setup incarnate files task to transfer input file to uspace
		IncarnateFiles incarnateFiles =
			new IncarnateFiles(
				"INCARNATEFILES",
				new ResourceSet(),
				filenames,
				contents,
				true);

		// get the encasulated AJO ResourceSet
		ResourceSet taskResourceSet = getResourceSet().getResourceSetClone();

		// add ApplicationResource that was set in checkContents
		// to ResourceSet
		taskResourceSet.add(getPreinstalledSoftware());

		// build task to execute a SoftwareResource
		UserTask executeTask = new UserTask(
			// task name
	getName(), null,
			// use ResourceSet that contains ApplicationResource
	taskResourceSet,
		getEnv(),
		getCommandLine(),
		null,
		getRedirectStdout(),
		getRedirectStderr(),
		isVerboseOn(),
		isVersionOn(),
		null,
		getMeasureTime(),
		getDebug(),
		getProfile());

		// build execute group and add UserTask to it
		executeGroup = new ActionGroup(getName() + "_EXECUTION");
		executeGroup.add(incarnateFiles);
		executeGroup.add(executeTask);

		try {
			executeGroup.addDependency(incarnateFiles, executeTask);
		} catch (InvalidDependencyException e) {
			logger.log(Level.SEVERE, "Cannot add dependency.", e);
		}

	}

	/**
	 *  Add container specific errors to error set of super classes and select
	 *  appropriate icon for display in Job preparation tree.
	 *
	 *@return    ErrorSet set containing all errors including those from
	 * super classes
	 */
	public ErrorSet checkContents() {
		ErrorSet err = super.checkContents();

		if (id == null || getName().equals("")) {
			err.add(new UError(getIdentifier(), "No task name specified"));
		}

		if (inputString == null || inputString.trim().length() == 0) {
			err.add(new UError(getIdentifier(), "No input file specified"));
		}

		// look for software resource if a vsite was selected
		if (getVsite() != null) {
			checkSoftwareResource(err);
		}

		setErrors(err);
		return err;
	}

	/**
	 * Get ResourceSet of currently selected Vsite from ResourceManager
	 * and look for our application resource. If we found a matching
	 * resource, store it in the field of super class UserContainer.
	 * If no matching resource was found add an error to the ErrorSet
	 * parameter
	 *
	 *@param  errorSet ErrorSet of container
	 */
	private void checkSoftwareResource(ErrorSet errorSet) {
		boolean softwareAvailable = false;
		// get Vsite ResourceSet
		ResourceSet siteResources =
			ResourceManager.getResourceSet(getVsite()).getResourceSet();
		// Iterate over set elements and check resource names
		ResourceEnumeration siteEnum = siteResources.elements();
		while (siteEnum.hasMoreElements()) {
			Resource siteResource = (Resource) siteEnum.nextElement();
			if (siteResource instanceof SoftwareResource) {
				SoftwareResource siteSoftware = (SoftwareResource) siteResource;
				// is this one our application?
				if (siteSoftware
					.getName()
					.toUpperCase()
					.startsWith(APPLICATION_NAME.toUpperCase())) {
					softwareAvailable = true;
					// store it in field of super class
					setPreinstalledSoftware(siteSoftware);
				}
			}
		}

		if (!softwareAvailable) {
			errorSet.add(
				new UError(
					getIdentifier(),
					"Application "
						+ APPLICATION_NAME
						+ " is not supported at site "
						+ getVsite()));
		}

	}

	/**
	 * Method of interface IPanelProvider:
	 * When our job is removed from Job Monitor, the additional outcome panel
	 * added from our plugin can perform some finalization.
	 */
	public void finalizePanel() {
		// nothing do be done here, for our example.
	}

	/**
	 * Method of interface IPanelProvider:
	 * Return number of additional outcome panels here.
	 *
	 * @return 1 for our example
	 */
	public int getNrOfPanels() {
		return 2;
	}

	/**
	 * Method of interface IPanelProvider:
	 * Get the ith additional outcome panel.
	 * Caution: do not build a new panel here for
	 * every method call, because every container
	 * object should contain exactly one instance of
	 * every additional outcome panel.
	 *
	 *@param  i  index of additional outcome panel
	 *@return    our outcome JPanel sub class
	 */
	public JPanel getPanel(int i) {
		if (i == 0) {
			if (samplePanel == null) {
				samplePanel = new BoltzmannSamplePanel();
			}
			return samplePanel;
		} 
		else {
			if (controlPanel == null) {
				controlPanel = new BoltzmannControlPanel();
			}
			return controlPanel;

		}
	}

	/**
	 * Method of interface IPanelProvider:
	 * Get a title for the ith addtional outcome panel. This will be
	 * used as header for the tabbed pane in the Job Monitor area.
	 *
	 *@param  i index of additional outcome panel
	 *@return    a nice title
	 */
	public String getPanelTitle(int i) {
		if(i==0) {
			return "Sample";
		}
		else {
			return "Control";
		}
	}	

	/**
	 *  Set a File object for the sript
	 *
	 *@param  inputFile  The new inputFile value
	 */
	public final void setInputFile(File inputFile) {
		this.inputFile = inputFile;
		if (inputFile == null) {
			return;
		}

		filename = inputFile.getName();
		int index = filename.indexOf(".");
		if (index > 0) {
			filename = filename.substring(0, index);
		}
	}

	/**
	 *  Set the script contents as String
	 *
	 *@param  inputString  The new inputString value
	 */
	public void setInputString(String inputString) {
		this.inputString = inputString;
	}

}
